@file:Suppress("ktlint:standard:filename")

package co.electriccoin.zcash.ui.screen.home

import androidx.activity.ComponentActivity
import androidx.activity.compose.BackHandler
import androidx.activity.viewModels
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.res.stringResource
import cash.z.ecc.android.sdk.model.ZecSend
import co.electriccoin.zcash.ui.MainActivity
import co.electriccoin.zcash.ui.R
import co.electriccoin.zcash.ui.common.viewmodel.HomeViewModel
import co.electriccoin.zcash.ui.screen.account.WrapAccount
import co.electriccoin.zcash.ui.screen.balances.WrapBalances
import co.electriccoin.zcash.ui.screen.home.model.TabItem
import co.electriccoin.zcash.ui.screen.home.view.Home
import co.electriccoin.zcash.ui.screen.receive.WrapReceive
import co.electriccoin.zcash.ui.screen.send.WrapSend
import co.electriccoin.zcash.ui.screen.send.model.SendArguments
import kotlinx.collections.immutable.persistentListOf
import kotlinx.coroutines.channels.BufferOverflow
import kotlinx.coroutines.flow.MutableSharedFlow

@Composable
@Suppress("LongParameterList")
internal fun MainActivity.WrapHome(
    onPageChange: (HomeScreenIndex) -> Unit,
    goBack: () -> Unit,
    goSettings: () -> Unit,
    goMultiTrxSubmissionFailure: () -> Unit,
    goScan: () -> Unit,
    goSendConfirmation: (ZecSend) -> Unit,
    sendArguments: SendArguments
) {
    WrapHome(
        this,
        onPageChange = onPageChange,
        goBack = goBack,
        goScan = goScan,
        goSendConfirmation = goSendConfirmation,
        goSettings = goSettings,
        goMultiTrxSubmissionFailure = goMultiTrxSubmissionFailure,
        sendArguments = sendArguments
    )
}

@Suppress("LongParameterList", "LongMethod")
@Composable
internal fun WrapHome(
    activity: ComponentActivity,
    goBack: () -> Unit,
    goSettings: () -> Unit,
    goMultiTrxSubmissionFailure: () -> Unit,
    goScan: () -> Unit,
    goSendConfirmation: (ZecSend) -> Unit,
    onPageChange: (HomeScreenIndex) -> Unit,
    sendArguments: SendArguments
) {
    val homeViewModel by activity.viewModels<HomeViewModel>()

    // Flow for propagating the new page index to the pager in the view layer
    val forceHomePageIndexFlow: MutableSharedFlow<ForcePage?> =
        MutableSharedFlow(
            Int.MAX_VALUE,
            Int.MAX_VALUE,
            BufferOverflow.SUSPEND
        )
    val forceIndex = forceHomePageIndexFlow.collectAsState(initial = null).value

    val homeGoBack: () -> Unit = {
        when (homeViewModel.screenIndex.value) {
            HomeScreenIndex.ACCOUNT -> goBack()
            HomeScreenIndex.SEND,
            HomeScreenIndex.RECEIVE,
            HomeScreenIndex.BALANCES -> forceHomePageIndexFlow.tryEmit(ForcePage(HomeScreenIndex.ACCOUNT))
        }
    }

    BackHandler {
        homeGoBack()
    }

    val tabs =
        persistentListOf(
            TabItem(
                index = HomeScreenIndex.ACCOUNT,
                title = stringResource(id = R.string.home_tab_account),
                testTag = HomeTag.TAB_ACCOUNT,
                screenContent = {
                    WrapAccount(
                        activity = activity,
                        goBalances = { forceHomePageIndexFlow.tryEmit(ForcePage(HomeScreenIndex.BALANCES)) },
                        goSettings = goSettings,
                    )
                }
            ),
            TabItem(
                index = HomeScreenIndex.SEND,
                title = stringResource(id = R.string.home_tab_send),
                testTag = HomeTag.TAB_SEND,
                screenContent = {
                    WrapSend(
                        activity = activity,
                        goToQrScanner = goScan,
                        goBack = homeGoBack,
                        goBalances = { forceHomePageIndexFlow.tryEmit(ForcePage(HomeScreenIndex.BALANCES)) },
                        goSendConfirmation = goSendConfirmation,
                        goSettings = goSettings,
                        sendArguments = sendArguments
                    )
                }
            ),
            TabItem(
                index = HomeScreenIndex.RECEIVE,
                title = stringResource(id = R.string.home_tab_receive),
                testTag = HomeTag.TAB_RECEIVE,
                screenContent = {
                    WrapReceive(
                        activity = activity,
                        onSettings = goSettings,
                    )
                }
            ),
            TabItem(
                index = HomeScreenIndex.BALANCES,
                title = stringResource(id = R.string.home_tab_balances),
                testTag = HomeTag.TAB_BALANCES,
                screenContent = {
                    WrapBalances(
                        activity = activity,
                        goSettings = goSettings,
                        goMultiTrxSubmissionFailure = goMultiTrxSubmissionFailure
                    )
                }
            )
        )

    Home(
        subScreens = tabs,
        forcePage = forceIndex,
        onPageChange = onPageChange
    )
}

/**
 * Wrapper class used to pass forced pages index into the view layer
 */
class ForcePage(
    val currentPage: HomeScreenIndex,
)

/**
 * Enum of the Home screen sub-screens
 */
enum class HomeScreenIndex {
    // WARN: Be careful when re-ordering these, as the ordinal number states for their order
    ACCOUNT,
    SEND,
    RECEIVE,
    BALANCES, ;

    companion object {
        fun fromIndex(index: Int) = entries[index]
    }
}
